// Copyright (C) 2001 Dominic Letourneau

#ifndef _WINSLEEP_CC_
#define _WINSLEEP_CC_

#include "Node.h"
#include "Object.h"
 
#include <csignal>
#include "BufferedNode.h"
#include "Buffer.h"
#include "Vector.h"
#include <windows.h>
#include <time.h>
#include <sys/types.h>
#include <sys/timeb.h>

using namespace std;

namespace FD {

class Sleep;

DECLARE_NODE(Sleep)
/*Node
 *
 * @name Sleep
 * @category Flow
 * @description Sleep a certain amount of time.
 *
 * @output_name VALUE
 * @output_description Always return TRUE.
 *
 * @parameter_name SECONDS
 * @parameter_description Sleep x seconds.
 * @parameter_type float
 *
END*/

bool recorder_stop_flag = false;

void signal_handler(int s)
{
	recorder_stop_flag = true;
}

class Sleep : public Node {
  
private:

  //int eofID;
  int outputID;
  struct _timeb tv;
  struct _timeb start_tv;

public:

   Sleep(string nodeName, ParameterSet params) 
      : Node(nodeName, params) 
   {
      
      float time;
      int time_sec;
      int time_msec;
      time = dereference_cast<float>(parameters.get("SECONDS"));
      time_sec = int(time);
      time_msec = int((time - time_sec)*1000);
      
      outputID = addOutput("VALUE");
      _ftime(&start_tv);
      start_tv.time = start_tv.time + time_sec;
      start_tv.millitm = start_tv.millitm + time_msec;
      
      if (start_tv.millitm >= 1000)
      {
		start_tv.millitm = start_tv.millitm - 1000;
		start_tv.time = start_tv.time + 1;
	  }
      signal(SIGINT, signal_handler);
   }
   
   
   
   ~Sleep() {}
   
   
   ObjectRef getOutput(int output_id, int count) 
   {      
   
     while(1)
     {
       //Sleep(1);
	   _ftime(&tv);

      if (output_id==outputID) 
      {
		if(recorder_stop_flag)
			return FalseObject;
		else
			if (tv.time >= start_tv.time && tv.millitm > start_tv.millitm)
			{
				return FalseObject;
			}
			else
				return TrueObject;
	  }
      else throw new NodeException (this, "Sleep: Unknown output id", __FILE__, __LINE__);
   	}
   }
   
 
 
};

}//namespace FD

#endif
